

class ListAllDeliveriesRequest:
    def __init__(self, pageNumber=None, limit=None):
        """ Initialize new instance from ListAllDeliveriesRequest class

        Parameters:
        pageNumber (int): Page Number
        limit (int): Number of deliveries returned from api

        Returns: New instance from ListAllDeliveriesRequest class
        """
        self.pageNumber = pageNumber
        self.limit = limit

    def toUrlQueryParamters(self):
        """
        Returns:
        JSON object from current instance

        """
        params = {}
        if self.pageNumber is not None:
            params["pageNumber"] = self.pageNumber
        if self.limit is not None:
            params["limit"] = self.limit
        return params





class ListAllPickupResponse:
    def __init__(self, res):
        """ Initialize new instance from ListAllPickupResponse class

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns: instance from ListAllPickupResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract pickups from json response object

        Parameters:
        res (dict, str): JSON response object or response text message

        """
        if type(res) is dict and res.get("data") is not None:
            self.pickups = res["data"]["pickups"]
            self.message = res["message"]
            self.success = res["success"]
        else:
            self.pickups = []
            self.message = str(res)

    def __str__(self):
        return self.message
    
    def get_pickups(self):
        return self.pickups

    def get_message(self):
        return self.message




import logging


class ListAllDeliveriesResponse:
    def __init__(self, res):
        """ Initialize new instance from ListAllDeliveriesResponse class

        Parameters:
        res (dict): JSON response object or response text message

        Returns: instance from ListAllDeliveriesResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract message, success, deliveries and
        count fields from json response object

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns:
        deliveries (array): List of deliveries
        count (int): Number of deliveries in list
        message (str): response message
        success (boolean) 
        """
        try:
            if type(res) is dict and res.get('data') is not None:
                self.message = res.get("message")
                self.success = res.get("success")
                self.count = res["data"]["count"]
                self.deliveries = []
                for delivery in res["data"]["deliveries"]:
                    self.deliveries.append(delivery)
            else:
                self.message = str(res)
                self.count = 0
                self.success = False
                self.deliveries = []

        except Exception as exp:
            raise exp

    def get_deliveries(self):
        return self.deliveries

    def get_count(self):
        return self.count
    def get_message(self):
        return self.message





class GetPickupDetailsResponse:

    def __init__(self, res):
        """
        Initialize new instance from GetPickupDetailsResponse class

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns:
        instance from GetPickupDetailsResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract _id, puid, business, businessLocationId,
        scheduledDate, scheduledTimeSlot, contactPerson,
        createdAt, deliveries, state, and tickets fields from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get("data") is not None:

            pickup = res["data"]
            self._id = pickup["_id"]
            self.puid = pickup["puid"]
            self.business = pickup["business"]
            self.scheduledDate = pickup["scheduledDate"]
            self.scheduledTimeSlot = pickup["scheduledTimeSlot"]
            self.contactPerson = pickup["contactPerson"]
            self.businessLocationId = pickup["businessLocationId"]
            self.deliveries = pickup["deliveries"]
            self.state = pickup["state"]
            self.createdAt = pickup["createdAt"]
            self.tickets = pickup["tickets"]
        else:
            self.message = str(res)

    def __str__(self):
        return self._id or self.message



DELIVERY_TYPES = {
    "PACKAGE_DELIVERY": {
        "code": 10,
        "value": "Forward"
    },
    "SEND": {
        "code": 10,
        "value": "Package Delivery"
    },
    "CASH_COLLECTION": {
        "code": 15,
        "value": "Cash Collection"
    },
    "RTO": {
        "code": 20,
        "value": "Return to Origin"
    },
    "CRP": {
        "code": 25,
        "value": "Customer Return Pickup"
    },
    "EXCHANGE": {
        "code": 30,
        "value": "Exchange"
    }
}





class GetDeliveryDetailsResponse:

    def __init__(self, res):
        """ Initialize new instance from GetDeliveryDetailsResponse class

        Parameters:
        res (dict, str): JSON Response object or response text message

        Returns: New instance from GetDeliveryDetailsResponse class
        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract _id, pickupAddress, dropOffAddress,
        cod, receiver, state, type, creationTimestamp,
        trackingNumber, holder, timeline,
        and history fields from json response object

        Parameters:
        jsonResponse (dict): JSON response object
        """
        if type(res) is dict and res.get('data') is not None:
            self.message = res.get("message")
            data = res["data"]
            self._id = data["_id"]
            self.pickupAddress = data["pickupAddress"]
            self.dropOffAddress = data["dropOffAddress"]
            self.cod = data["cod"]
            self.receiver = data["receiver"]
            self.state = data["state"]
            self.type = data["type"]
            self.trackingNumber = data["trackingNumber"]
            self.holder = data["holder"]
            self.timeline = data["timeline"]
            self.history = data["history"]
            self.creationTimestamp = data["creationTimestamp"]
        else:
            self.message = str(res)

    def __str__(self):
        return self.message
    def get_message(self):
        return self.message
    def get_deliveryId(self):
        return self._id



class Address:
    def __init__(
        self, cityCode: str, firstLine: str, zone=None,
        secondLine=None, district=None, buildingNumber=None,
        floor=None, apartment=None
    ):
        """ Initialize new instance from Address class

        Parameters:
        cityCode (str)
        zone (str)
        secondLine (str)
        district (str)
        buildingNumber (int)
        floor (int)
        apartment (int)

        Returns: New instance from Address class
        """
        self.cityCode = cityCode
        self.firstLine = firstLine
        self.zone = zone
        self.secondLine = secondLine
        self.district = district
        self.buildingNumber = buildingNumber
        self.floor = floor
        self.apartment = apartment

    def get_firstLine(self):
        return self.firstLine

    def get_secondLine(self):
        return self.secondLine

    def get_city(self):
        return self.cityCode

    def get_zone(self):
        return self.zone

    def toJSON(self):
        """
        Returns:
        JSON object from current instance

        """
        addressObj = {
            "cityCode": self.cityCode,
            "firstLine": self.firstLine
        }
        if self.zone is not None:
            addressObj["zone"] = self.zone
        if self.secondLine is not None:
            addressObj["secondLine"] = self.secondLine
        if self.district is not None:
            addressObj["district"] = self.district
        if self.buildingNumber is not None:
            addressObj["buildingNumber"] = self.buildingNumber
        if self.floor is not None:
            addressObj["floor"] = self.floor
        if self.apartment is not None:
            addressObj["apartment"] = self.apartment

        return addressObj


if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))




from .Address import Address
from .Receiver import Receiver
from .DeliverySpecs import DeliverySpecs
from .ContactPerson import ContactPerson
from .DeliveryTypes import DELIVERY_TYPES
from .PickupTimeSlots import PICKUP_TIME_SLOTS





class CreateDeliveryResponse:
    def __init__(self, res):
        """
        Initialize new instance from CreateDeliveryResponse class

        Parameters:
        res (dict): JSON response object or response text message

        Returns:
        instance from CreateDeliveryResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract _id, trackingNumber and message fields from json response object

        Parameters:
        res (dict): JSON response object or response text message

        Returns:
        _id (str): Delivery Id
        trackingNumber (str): Delivery trackingNumber
        message (str): Create delivery response message
        """
        if type(res) is dict and res.get('data') is not None:
            self._id = res["data"]["_id"]
            self.trackingNumber = res["data"]["trackingNumber"]
            self.message = res["data"]["message"]
        else:
            self.message = str(res)
            self._id = self.trackingNumber = None


    def get_deliveryId(self):
        return self._id

    def get_trackingNumber(self):
        return self.trackingNumber

    def get_message(self):
        return self.message

    def __str__(self):
        return self.message




import os
import json
import logging

from bostaSDK.utils.Address import Address
from bostaSDK.utils.Receiver import Receiver
from bostaSDK.utils.DeliverySpecs import DeliverySpecs


from .list.ListAllDeliveriesRequest import ListAllDeliveriesRequest
from .list.ListAllDeliveriesResponse import ListAllDeliveriesResponse
from .get.GetDeliveryDetailsRequest import GetDeliveryDetailsRequest
from .get.GetDeliveryDetailsResponse import GetDeliveryDetailsResponse
from .update.UpdateDeliveryRequest import UpdateDeliveryRequest
from .update.UpdateDeliveryResponse import UpdateDeliveryResponse
from .printAWB.PrintAWBRequest import PrintAWBRequest
from .printAWB.PrintAWBResponse import PrintAWBResponse
from .terminate.TerminateDeliveryRequest import TerminateDeliveryRequest
from .terminate.TerminateDeliveryResponse import TerminateDeliveryResponse
from .create.CreateDeliveryRequest import CreateDeliveryRequest
from .create.CreateDeliveryResponse import CreateDeliveryResponse
from .track.TrackDeliveryRequest import TrackDeliveryRequest
from .track.TrackDeliveryResponse import TrackDeliveryResponse


class Delivery:

    def __init__(self, apiClient):
        self.apiClient = apiClient

    def listAll(self, listAllDeliveriesRequest: ListAllDeliveriesRequest) -> ListAllDeliveriesResponse:
        """
        List All Deliveries.

        Parameters:
        listAllDeliveriesRequest (ListAllDeliveriesRequest)

        Returns: New instance from ListAllDeliveriesResponse.
        """
        try:
            logging.info("list all business deliveries")
            url = "/deliveries"
            params = listAllDeliveriesRequest.toUrlQueryParamters()
            response = self.apiClient.send('get', url, params=params)
            if (response.status_code) != 200:
                return ListAllDeliveriesResponse(response.text)
            instance = ListAllDeliveriesResponse(response.json())
            return instance
        except Exception as exp:
            logging.error(exp)
            raise exp

    def get(self, getDeliveryDetailsRequest: GetDeliveryDetailsRequest) -> GetDeliveryDetailsResponse:
        """
        Get Delivery.

        Parameters:
        getDeliveryDetailsRequest (GetDeliveryDetailsRequest)

        Returns: New instance from GetDeliveryDetailsResponse.
        """
        try:
            logging.info('Get Delivery')
            url = "/deliveries/" + \
                str(getDeliveryDetailsRequest.get_deliveryId())
            response = self.apiClient.send('get', url)
            if (response.status_code) != 200:
                return GetDeliveryDetailsResponse(response.text)
            return GetDeliveryDetailsResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def printAirWayBill(self, printAWBRequest: PrintAWBRequest) -> PrintAWBResponse:
        """
        Print Delivery Airway Bill.

        Parameters:
        printAWBRequest (PrintAWBRequest)

        Returns: New instance from PrintAWBResponse.
        """
        try:
            logging.info("Print airway bill")
            url = "/deliveries/awb" +  str(printAWBRequest.get_deliveryId())
            response = self.apiClient.send('get', url)
            if (response.status_code) != 200:
                return PrintAWBResponse(response.text)
            return PrintAWBResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def create(self, createDeliveryRequest: CreateDeliveryRequest) -> CreateDeliveryResponse:
        """
        Create New Delivery.

        Parameters:
        createDeliveryRequest (CreateDeliveryRequest)

        Returns: New instance from CreateDeliveryResponse.
        """
        try:
            payload = createDeliveryRequest.toJSONPayload()
            logging.info('Create New Delivery: Payload ' + str(payload))
            url = "/deliveries"
            headers = {
                "content-type": "application/json"
            }
            response = self.apiClient.send(
                'post', url, headers=headers, data=payload)
            if (response.status_code) != 201 or (response.status_code) != 200:
                return CreateDeliveryResponse(response.text)
            return CreateDeliveryResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def update(self, updateDeliveryRequest: UpdateDeliveryRequest) -> UpdateDeliveryResponse:
        """
        Update Delivery.

        Parameters:
        updateDeliveryRequest (UpdateDeliveryRequest)

        Returns: New instance from UpdateDeliveryResponse.
        """
        try:
            logging.info('Update Delivery')
            url = "/deliveries/" + str(updateDeliveryRequest.get_deliveryId())
            payload = updateDeliveryRequest.toJSONPayload()
            response = self.apiClient.send('patch', url, data=payload)
            if (response.status_code) != 200:
                return UpdateDeliveryResponse(response.text)
            return UpdateDeliveryResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def terminate(self, terminateDeliveryRequest: TerminateDeliveryRequest) -> TerminateDeliveryResponse:
        """
        Terminate Delivery.

        Parameters:
        terminateDeliveryRequest (TerminateDeliveryRequest)

        Returns: New instance from TerminateDeliveryResponse.
        """
        try:
            logging.info("Terminate Delivery")
            url = "/deliveries/" + \
                str(terminateDeliveryRequest.get_deliveryId())
            response = self.apiClient.send('delete', url)
            if (response.status_code) != 200:
                return TerminateDeliveryResponse(response.text)
            return TerminateDeliveryResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def track(self, trackDeliveryRequest: TrackDeliveryRequest) -> TrackDeliveryResponse:
        """
        Track Delivery.

        Parameters:
        trackDeliveryRequest (TrackDeliveryRequest)

        Returns: New instance from TrackDeliveryResponse.
        """
        try:
            logging.info('Track Delivery')
            url = "/deliveries/" + \
                str(trackDeliveryRequest.get_deliveryId()) + "/tracking"
            response = self.apiClient.send('get', url)
            if (response.status_code) != 200:
                return TrackDeliveryResponse(response.text)
            return TrackDeliveryResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp



import os
import json
import logging

from bostaSDK.apiClient.ApiClient import ApiClient


class BusinessInfo:
    def __init__(self, apiClient):
        """
        Initialize new instance from BusinessInfo class

        Parameters:
        apiClient:  ApiClient

        Returns:
        BusinessInfo: new instance from BusinessInfo

        """
        self.apiClient = apiClient

    def getBusinessSubAccount(self, pageId=0):
        """
        List all business sub accounts

        Parameters:
        pageId (int, optinal): the sequence of pages

        Returns:
        list of all sub accounts
        """
        try:
            logging.info('Get Business Sub Account')
            url = "/businessSubAccount"
            params = {
                "pageId": pageId
            }
            response = self.apiClient.send('get', url, params)
            if (response.status_code != 200):
                return response.text
            return response.json()
        except Exception as exp:
            logging.error(exp)
            raise exp




class TerminateDeliveryResponse:

    def __init__(self, res):
        """ Initialize new instance from TerminateDeliveryResponse class

        Parameters:
        res (dict, str): JSON Response object or response text message

        Returns: New instance from TerminateDeliveryResponse class
        """
        self.fromResposeObj(res)

    def fromResposeObj(self, res):
        """
        Extract message field from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get("message") is not None:
            self.message = res["message"]
        else:
            self.message = str(res)

    def __str__(self):
        return self.message



class City:
    def __init__(self, apiClient):
        self.apiClient = apiClient

    def listAll(self):
        """ List all citites

        Returns: List of available cities
        """
        try:
            headers = {
                "accept": "application/json"
            }
            response = self.apiClient.send(
                'get',
                "/cities",
                headers
            )
            if (response.status_code != 200):
                return response.text
            return response.json()
        except Exception as exp:
            raise exp




import os
import logging


from .list.ListAllPickupsRequest import ListAllPickupsRequest
from .list.ListAllPickupsResponse import ListAllPickupResponse

from .create.CreatePickupRequest import CreatePickupRequest
from .create.CreatePickupResponse import CreatePickupResponse

from .get.GetPickupDetailsRequest import GetPickupDetailsRequest
from .get.GetPickupDetailsResponse import GetPickupDetailsResponse

from .update.UpdatePickupRequest import UpdatePickupRequest
from .update.UpdatePickupResponse import UpdatePickupResponse

from .delete.DeletePickupRequest import DeletePickupRequest
from .delete.DeletePickupResponse import DeletePickupResonse


class Pickup:
    def __init__(self, apiClient):
        self.apiClient = apiClient

    def create(self, createPickupRequest: CreatePickupRequest) -> CreatePickupResponse:
        """
        Create New Pickup.

        Parameters:
        createPickupRequest (CreatePickupRequest)

        Returns: New instance from CreatePickupResponse.
        """
        try:
            logging.info("Create New Pickup")
            url = "/pickups"
            headers = {
                "content-type": "application/json"
            }
            response = self.apiClient.send(
                'post', url, headers=headers, data=createPickupRequest.toJSONPayload())
            if (response.status_code != 200):
                return CreatePickupResponse(response.text)
            return CreatePickupResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def update(self, updatePickupRequest: UpdatePickupRequest) -> UpdatePickupResponse:
        """
        Update Pickup.

        Parameters:
        updatePickupRequest (UpdatePickupRequest)

        Returns: New instance from UpdatePickupResponse.
        """
        try:
            logging.info("Update Pickup")
            url = "/pickups/" + str(updatePickupRequest.get_id())
            response = self.apiClient.send(
                'put',
                url,
                data=updatePickupRequest.toJSONPayload()
            )
            if (response.status_code != 200):
                return UpdatePickupResponse(response.text)
            return UpdatePickupResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def listAll(self, listAllPickupsRequest: ListAllPickupsRequest) -> ListAllPickupResponse:
        """
        List All Pickup.

        Parameters:
        listAllPickupsRequest (ListAllPickupsRequest)

        Returns: New instance from ListAllPickupResponse.
        """
        try:
            url = "/pickups"
            params = listAllPickupsRequest.toQueryParamters()
            response = self.apiClient.send('get', url, params=params)
            if (response.status_code != 200):
                return ListAllPickupResponse(response.text)
            return ListAllPickupResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def get(self, getPickupDetailsRequest: GetPickupDetailsRequest) -> GetPickupDetailsResponse:
        """
        Get Pickup.

        Parameters:
        getPickupDetailsRequest (GetPickupDetailsRequest)

        Returns: New instance from GetPickupDetailsResponse.
        """
        try:
            logging.info("Get Pickup")
            url = "/pickups/" + str(getPickupDetailsRequest.get_pickupId())
            response = self.apiClient.send('get', url)
            if (response.status_code != 200):
                return GetPickupDetailsResponse(response.text)
            return GetPickupDetailsResponse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp

    def delete(self, deletePickupRequest: DeletePickupRequest) -> DeletePickupResonse:
        """
        Delete Pickup.

        Parameters:
        deletePickupRequest (DeletePickupRequest)

        Returns: New instance from DeletePickupResonse.
        """
        try:
            logging.info("Delete Pickup")
            url = "/pickups/" + str(deletePickupRequest.get_pickupId())
            response = self.apiClient.send('delete', url)
            if (response.status_code != 200):
                return DeletePickupResonse(response.text)
            return DeletePickupResonse(response.json())
        except Exception as exp:
            logging.error(exp)
            raise exp




from .Pickup import Pickup

from .create.CreatePickupRequest import CreatePickupRequest
from .create.CreatePickupResponse import CreatePickupResponse

from .list.ListAllPickupsRequest import ListAllPickupsRequest
from .list.ListAllPickupsResponse import ListAllPickupResponse

from .get.GetPickupDetailsRequest import GetPickupDetailsRequest
from .get.GetPickupDetailsResponse import GetPickupDetailsResponse

from .update.UpdatePickupRequest import UpdatePickupRequest
from .update.UpdatePickupResponse import UpdatePickupResponse

from .delete.DeletePickupRequest import DeletePickupRequest
from .delete.DeletePickupResponse import DeletePickupResonse




class PrintAWBResponse:
    def __init__(self, res):
        """ Initialize new instance from PrintAWBResponse class

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns: instance from PrintAWBResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract data field from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get("data") is not None:
            self.data = res["data"]
            self.message = res.get("message")
        else:
            self.message = str(res)
            self.data = None

    def __str__(self):
        return self.message
    
    def get_data(self):
        return self.data



import json

from bostaSDK.utils.ContactPerson import ContactPerson


class CreatePickupRequest:

    def __init__(self, scheduledDate: str,
                 scheduledTimeSlot: str, contactPerson: ContactPerson,
                 businessId=None, businessLocationId=None,
                 warehouseId=None, noOfPackages=None, notes=None):
        """ Initialize new instance from CreatePickupRequest class

        Parameters:
        scheduledDate (str): Pickup scheduled date
        scheduledTimeSlot (str): "10:00 to 13:00" or "13:00 to 16:00"
        contactPerson (ContactPerson)
        businessId (str)
        businessLocationId (str)
        warehouseId (str)
        noOfPackages (int)
        notes (str)

        Returns: instance from CreatePickupRequest
        """

        self.businessId = businessId
        self.scheduledDate = scheduledDate
        self.scheduledTimeSlot = scheduledTimeSlot
        self.businessLocationId = businessLocationId
        self.contactPerson = contactPerson
        self.noOfPackages = noOfPackages
        self.notes = notes
        self.warehouseId = warehouseId

    def toJSONPayload(self):
        """
        Returns:
        JSON object from current instance

        """
        payload = {
            "scheduledDate": self.scheduledDate,
            "scheduledTimeSlot": self.scheduledTimeSlot,
            "contactPerson": self.contactPerson.toJSON(),
        }
        if self.businessId is not None:
            payload['businessId'] = self.businessId
        if self.businessLocationId is not None:
            payload['businessLocationId'] = self.businessLocationId
        if self.noOfPackages is not None:
            payload['noOfPackages'] = self.noOfPackages
        if self.notes is not None:
            payload['notes'] = self.notes
        if self.warehouseId is not None:
            payload['warehouseId'] = self.warehouseId
        return json.dumps(payload)





class ListAllPickupsRequest:

    def __init__(self, pageId, sortBy=None, sortValue=None):
        """ Initialize new instance from ListAllPickupsRequest class

        Parameters:
        pageId (int)
        sortBy (str)
        sortValue (str)

        Returns: New instance from ListAllPickupsRequest class
        """
        self.pageId = pageId
        self.sortBy = sortBy
        self.sortValue = sortValue

    def toQueryParamters(self):
        """
        Returns:
        JSON object from current instance

        """
        params = {
            "pageId": self.pageId,
        }
        if self.sortBy is not None:
            params["sortBy"] = self.sortBy
        if self.sortValue is not None:
            params["sortValue"] = self.sortValue
        return params



class DeliverySpecs:
    def __init__(
        self, size: str, weight=None, length=None, height=None,
        itemsCount=None, description=None, document=None
    ):
        """ Initialize new instance from DeliverySpecs class

        Parameters:
        size (str)
        weight (float)
        length (float)
        height (float)
        itemsCount (int)
        description (str)
        document (str)

        Returns: New instance from DeliverySpecs class
        """
        self.size = size
        self.weight = weight
        self.packageDetails = {
            "itemsCount": itemsCount,
            "description": description,
            "document": "document"
        }
        self.dimensions = {
            "length": length,
            "weight": weight,
            "height": height
        }

    def get_size(self, size):
        return self.size

    def get_weight(self, weight):
        return self.weight

    def get_packageDetails(self, packageDetails):
        return self.packageDetails

    def get_dimensions(self, dimensions):
        return self.dimensions

    def toJSON(self):
        """
        Returns:
        JSON object from current instance

        """
        specs = { 
            "size": self.size
            }
        if self.weight is not None:
            specs['weight'] = self.weight
        if self.packageDetails['itemsCount'] is not None:
            specs['packageDetails'] = self.packageDetails
        if self.dimensions['length'] is not None:
            specs['dimensions'] = self.dimensions
        return specs

    def __str__(self):
        return str(self.size, self.weight)


if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))





class TerminateDeliveryRequest:
    def __init__(self, deliveryId):
        """ Initialize new instance from TerminateDeliveryRequest class

        Parameters:
        deliveryId (str): Delivery Id

        Returns: New instance from TerminateDeliveryRequest class
        """
        self._id = deliveryId

    def get_deliveryId(self):
        return self._id

    def __str__(self): return self._id



import json

from bostaSDK.utils.Address import Address
from bostaSDK.utils.Receiver import Receiver


class UpdateDeliveryRequest:
    def __init__(
        self, deliveryId: int, receiver=None,
        pickUpAddress=None, dropOffAddress=None,
        returnAddress=None, notes=None,
        businessReference=None,
        webhookUrl=None, cod=None
    ):
        """ Initialize new instance from UpdateDeliveryRequest class

        Parameters:
        deliveryId (str): Delivery Id
        specs (DeliverySpecs)
        pickUpAddress (Address, optinal)
        dropOffAddress (Address, optinal)
        returnAddress (Address, optinal)
        cod (int, optinal): Cash on delivery amount
        receiver (Receiver, optinal)
        businessReference (str, optinal): Business refrence
        webhookUrl (str, optinal)

        Returns: instance from UpdateDeliveryRequest

        """
        self._id = deliveryId
        self.receiver = receiver
        self.pickUpAddress = pickUpAddress
        self.dropOffAddress = dropOffAddress
        self.returnAddress = returnAddress
        self.notes = notes
        self.businessReference = businessReference
        self.webhookUrl = webhookUrl
        self.cod = cod

    def get_deliveryId(self):
        return self._id

    def toJSONPayload(self):
        """
        Returns:
        JSON object from current instance

        """
        payload = {}
        if self.receiver is not None:
            payload['receiver'] = self.receiver.toJSON()
        if self.pickUpAddress is not None:
            payload['pickUpAddress'] = self.pickUpAddress.toJSON()
        if self.dropOffAddress is not None:
            payload['dropOffAddress'] = self.dropOffAddress.toJSON()
        if self.returnAddress is not None:
            payload['returnAddress'] = self.returnAddress.toJSON()
        if self.notes is not None:
            payload['notes'] = self.notes
        if self.businessReference is not None:
            payload['businessReference'] = self.businessReference
        if self.webhookUrl is not None:
            payload['webhookUrl'] = self.webhookUrl
        if self.cod is not None:
            payload['cod'] = self.cod
        return json.dumps(payload)



import os

from bostaSDK.apiClient import ApiClient
from bostaSDK.delivery import list, create, update, track, printAWB, get, terminate
from bostaSDK.utils import Receiver, Address, ContactPerson

apiKey = os.getenv("BOSTA-API-KEY")
baseUrl = os.getenv("BOSTA-BASE-URL")

apiClient = ApiClient(apiKey, baseUrl)
deliveryTypes = apiClient.deliveyTypes

#List Deliveries 
apiClient.delivery.listAll(list.ListAllDeliveriesRequest(3,10))

#Create new delivery
reciever = Receiver("sohila", "Boghdady", "sohila_bogdady@hotmail.com", "01090055000")
dropOffAddress = Address("EG-01", "Maadi", "Maadi", "104")
createDeliveryReq = create.CreateDeliveryRequest(
    deliveryTypes['SEND']['code'], 100, dropOffAddress, reciever 
)
deliveryId = apiClient.delivery.create(createDeliveryReq)

#Print Airway Bill
apiClient.delivery.printAirWayBill(printAWB.PrintAWBRequest([deliveryId]))

#Track Delivery
apiClient.delivery.track(track.TrackDeliveryRequest(deliveryId))

#Get Delivery
apiClient.delivery.get(get.GetDeliveryDetailsRequest(deliveryId))

#Update Delivery
newReciever = Receiver("user", "test", "test@example.com", "01090055000")
apiClient.delivery.update(update.UpdateDeliveryRequest(deliveryId, receiver = newReciever, cod=120))

#Terminate Delivery
apiClient.delivery.terminate(terminate.TerminateDeliveryRequest(deliveryId))




class GetPickupDetailsRequest:
    def __init__(self, pickupId):
        """ Initialize new instance from GetPickupDetailsRequest class

        Parameters:
        pickupId (str): Pickup Id

        Returns: instance from GetPickupDetailsRequest

        """
        self._id = pickupId

    def get_pickupId(self):
        return self._id

    def __str__(self):
        return str(self._id)




class UpdateDeliveryResponse:

    def __init__(self, res):
        """
        Initialize new instance from UpdateDeliveryResponse class

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns:
        instance from UpdateDeliveryResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract _id and message fields from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get("_id") is not None:
            self.message = res["message"]
            self._id = res["_id"]
        else:
            self.message = str(res)

    def __str__(self):
        return self.message
    
    def get_message(self):
        return self.message





class TrackDeliveryResponse:

    def __init__(self, res):
        """ Initialize new instance from TrackDeliveryResponse class

        Parameters:
        res (dict, str): JSON Response object or response text message

        Returns: New instance from TrackDeliveryResponse class
        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract message field from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get("data") is not None:
            self.message = res["message"]
            self.success = res["success"]
            self.state_history = []
            for obj in res["data"]["state-history"]:
                self.state_history.append({
                    "state": obj["state"],
                    "time": obj["timestamp"],
                    "takenBy": obj["takenBy"]["userName"]
                })
            self._id = res["data"]["_id"]
            self.trackingNumber = res["data"]["trackingNumber"]
        else:
            self.message = str(res)

    def __str__(self):
        return str(self._id) or self.message
    def get_message(self):
        return self.message 
    def get_state_history(self):
        return self.state_history or None






class TrackDeliveryRequest:
    def __init__(self, deliveryId):
        """ Initialize new instance from TrackDeliveryRequest class

        Parameters:
        deliveryId (str): Delivery id

        Returns: New instance from TrackDeliveryRequest class
        """
        self._id = deliveryId

    def __str__(self):
        return str(self._id)

    def get_deliveryId(self):
        return self._id



from setuptools import setup, Extension, find_packages

setup(
    name='bostaSDK',
    version='1.8.0',
    author='bosta',
    author_email='sdk@bosta.com',
    scripts=[],
    packages = find_packages(),
    keywords=["bosta", "client-sdk", "api", "api-integration"],
    url= 'https://pypi.org/project/bostaSDK/',
    license='LICENSE',
    description='Bosta Python SDK',
    long_description=open('README.md', 'r').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        'requests'
    ],
    python_requires='>=3.6',
)





class CreatePickupResponse:

    def __init__(self, res):
        """
        Initialize new instance from CreatePickupResponse class

        Parameters:
        res (dict, str): JSON response object or response text message

        Returns:
        instance from CreatePickupResponse

        """
        self.fromResponseObj(res)

    def fromResponseObj(self, res):
        """
        Extract _id, puid, business, businessLocationId,
        scheduledDate, scheduledTimeSlot, contactPerson,
        createdAt and updatedAt fields from json response object

        Parameters:
        res (dict, str): JSON response object or response text message
        """
        if type(res) is dict and res.get('data') is not None:
            self.message = res.get("message")
            newPickup = res["data"]
            self._id = newPickup["_id"]
            self.puid = newPickup["puid"]
            self.business = newPickup["business"]
            self.businessLocationId = newPickup["businessLocationId"]
            self.scheduledDate = newPickup["scheduledDate"]
            self.scheduledTimeSlot = newPickup["scheduledTimeSlot"]
            self.contactPerson = newPickup["contactPerson"]
            self.createdAt = newPickup["createdAt"]
            self.updatedAt = newPickup["updatedAt"]
        else:
            self.message = str(res)

    def __str__(self):
        return self.message
    
    def get_pickupId(self):
        return self._id
    



import os 

from bostaSDK.apiClient import ApiClient
from bostaSDK.utils import ContactPerson
from bostaSDK.pickup import create, list, get, update, delete

apiKey = os.getenv("BOSTA-API-KEY")
baseUrl = os.getenv("BOSTA-BASE-URL")
apiClient = ApiClient(apiKey, baseUrl)

# Create New Pickup
newPickupId = apiClient.pickup.create(
    create.CreatePickupRequest(
     "Mon Nov 7 2020 00:00:00 GMT+0200", apiClient.pickupTimeSlots[0],
      ContactPerson ("m", "010193155997", "example@bosta.co")
    )
)

#List Pickups 
apiClient.pickup.listAll(list.ListAllPickupsRequest(2))

#Get Pickup 
apiClient.pickup.get(
    get.GetPickupDetailsRequest(newPickupId)
)

#Update Pickup 
apiClient.pickup.update(update.UpdatePickupRequest(
    newPickupId, contactPerson=ContactPerson ("user", "010193155922", "user@bosta.co")
))

#Delete Pickup
apiClient.pickup.delete(delete.DeletePickupRequest(newPickupId))


class Receiver:
    def __init__(self, firstName, lastName, email, phone):
        """ Initialize new instance from Receiver class

        Parameters:
        firstName (str)
        lastName (str)
        email (str)
        phone (str)

        Returns: New instance from Receiver class
        """
        self.firstName = firstName
        self.lastName = lastName
        self.email = email
        self.phone = phone

    def get_firstName(self):
        return self.firstName

    def get_lastName(self):
        return self.lastName

    def get_email(self):
        return self.email

    def get_phone(self):
        return self.phone

    def toJSON(self):
        """
        Returns:
        JSON object from current instance

        """
        return {
            "firstName": self.firstName,
            "lastName": self.lastName,
            "email": self.email,
            "phone": self.phone
        }


if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))



import json


class UpdatePickupRequest:
    def __init__(
            self, _id, businessId=None, scheduledDate=None,
            scheduledTimeSlot=None, contactPerson=None,
            businessLocationId=None, warehouseId=None,
            noOfPackages=None, notes=None):
        """ Initialize new instance from UpdatePickupRequest class

        Parameters:
        _id (str): Pickup Id
        businessId (str)
        scheduledDate (str)
        scheduledTimeSlot (str)
        contactPerson (ContactPerson)
        businessLocationId (str)
        warehouseId (str)
        noOfPackages (str)
        notes (str)


        Returns: instance from UpdatePickupRequest

        """
        self._id = _id
        self.businessId = businessId
        self.scheduledDate = scheduledDate
        self.scheduledTimeSlot = scheduledTimeSlot
        self.businessLocationId = businessLocationId
        self.contactPerson = contactPerson
        self.noOfPackages = noOfPackages
        self.notes = notes
        self.warehouseId = warehouseId

    def get_id(self):
        return self._id

    def toJSONPayload(self):
        """
        Returns:
        JSON object from current instance

        """
        payload = {}
        if self.scheduledDate is not None:
            payload['scheduledDate'] = self.scheduledDate
        if self.scheduledTimeSlot is not None:
            payload['scheduledTimeSlot'] = self.scheduledTimeSlot
        if self.contactPerson is not None:
            payload['contactPerson'] = self.contactPerson.toJSON()
        if self.businessId is not None:
            payload['businessId'] = self.businessId
        if self.businessLocationId is not None:
            payload['businessLocationId'] = self.businessLocationId
        if self.noOfPackages is not None:
            payload['noOfPackages'] = self.noOfPackages
        if self.notes is not None:
            payload['notes'] = self.notes
        if self.warehouseId is not None:
            payload['warehouseId'] = self.warehouseId
        return json.dumps(payload)



import json

from bostaSDK.utils.Address import Address
from bostaSDK.utils.Receiver import Receiver
from bostaSDK.utils.DeliverySpecs import DeliverySpecs


class CreateDeliveryRequest:

    def __init__(
        self, deliveryType, cod,
        dropOffAddress: Address, receiver: Receiver,
        deliverySpecs=None, notes=None,
        businessReference=None,
        ):
        """ Initialize new instance from CreateDeliveryRequest class

        Parameters:
        deliveryType (int)
        cod (int): Cash on delivery amount
        dropOffAddress (Address)
        cashOnDelivery (int): Cash on delivery amount
        receiver (Receiver)
        deliverySpecs (DeliverySpecs)
        notes (str)
        businessReference (str)

        Returns: instance from CreateDeliveryRequest

        """
        self.type = deliveryType
        self.cod = cod
        self.dropOffAddress = dropOffAddress
        self.receiver = receiver

    def toJSONPayload(self):
        """
        Returns:
        JSON object from current instance

        """
        return json.dumps({
            "type": self.type,
            "cod": self.cod,
            "dropOffAddress": self.dropOffAddress.toJSON(),
            "receiver": self.receiver.toJSON()
        })





class PrintAWBRequest:
    def __init__(self, deliveryId):
        """ Initialize new instance from PrintAWBRequest class

        Parameters:
        deliveryId (str): Delivery Id

        Returns: instance from PrintAWBRequest

        """
        self._id = deliveryId

    def __str__(self):
        return str(self._id)

    def get_deliveryId(self):
        return self._id



# Copyright 2020 by BOSTA. All Rights Reserved.
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation, and that the name of bosta
# not be used in advertising or publicity pertaining to distribution
# of the software without specific, written prior permission.
# BOSTA DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
# ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL
# BOSTA BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
# ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
# IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT
# OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

"""
Bosta package for Python. Based on PEP 282 and comments thereto in
comp.lang.python.

Copyright (C) 2020. All Rights Reserved.

To use, simply 'import bostaSDK'!
"""

from .delivery import *
from .pickup import *
from .utils import *

__author__ = "sohila bogdady <sdk@bosta.co>"
__status__ = "development"
__version__ = "0.9"
__date__ = "07 Sep 2020"
__name__ = "Bosta"



import requests

from ..delivery import Delivery
from ..pickup import Pickup

from ..city import City
from ..zone import Zone

from ..utils import DELIVERY_TYPES
from ..utils import PICKUP_TIME_SLOTS


class ApiClient:
    def __init__(self, apiKey, apiBase="https://app.bosta.co"):
        """
        Initialize new instance from apiClient class

        Parameters:
        apiKey (string): Business api key
        apiBase (string): Bosta host url

        Returns:
        apiClient: instance from apiClient

        """
        self.apiKey = apiKey
        self.apiBase = apiBase
        self.pickup = Pickup(self)
        self.delivery = Delivery(self)
        self.cities = City(self).listAll()
        self.zones = Zone(self).listAll()
        self.deliveyTypes = DELIVERY_TYPES
        self.pickupTimeSlots = PICKUP_TIME_SLOTS

    def get_apiKey(self): return self.apiKey
    def get_apiBase(self): return self.apiBase
    def get_delivery(self): return self.delivery
    def get_pickup(self): return self.pickup
    def get_cities(self): return self.cities
    def get_zones(self): return self.zones

    def send(self, method, url, headers={}, params=None, data=None):
        try:
            """
            Send HTTP Request

            Parameters:
            method (string): Request method type
            url (string): Full url path
            headers (dict): Request headers
            params (dict): Request parameters
            data (dict): Request payload data

            Returns:
            response: Http response object

            """
            headers['X-Requested-By'] = 'python-sdk'
            headers['Authorization'] = self.apiKey
            url = self.apiBase + "/api/v1" + url
            return getattr(requests, method)(
                url, headers=headers, params=params, data=data
            )
        except Exception as exp:
            raise exp


if __name__ == '__main__' and __package__ is None:
    from os import sys, path
    sys.path.append(path.dirname(path.dirname(path.abspath(__file__))))
